#!/bin/bash
########################################################################
# build_git.sh
#
# Build git and package for UPS.
#
# Adapted from build.sh.template provided by ssibuildshims v2_16_01.
########################################################################

########################################################################
# Non-customize-able boilerplate: skip to "Customize-able functions."
########################################################################

####################################
# Useful variables.
prog=${BASH_SOURCE[0]##*/}

##################
# Local (dumb) ssi_die() function for use until we source ssi_functions.
ssi_die() {
  printf "FATAL ERROR: $*\n" 1>&2; exit 1
}
##################

##################
# Basic usage function for use by ensure_buildshims until we have enough
# information to provide something more specific. Do NOT customize!
build_script_usage() {
    cat <<EOF
USAGE: $prog <ssi-msg-config-options> [--] <product-dir> <qualifier-args> <maketar>
       $prog -h|--help [<product-dir>]

  Build $package and package for UPS.

OPTIONS

  -h|--help

    Help. Supply <product-dir> for specific, detailed information on
    invoking $prog.

  <ssi-msg-config-options>

    Supply <product-dir> for more information.

ARGUMENTS

  <product-dir>

    The top-level UPS directory in which the source code distribution
    was unpacked. When specified with -h|--help, this enables us to
    provide more specific help about invoking $prog.

  <qualifier-args>
  <maketar>

    Supply <product-dir> for more information.

EOF
}
##################

##################
# Make sure we have UPS and ssibuildshims set up.
ensure_ssibuildshims() {
  if ! (( $# )) || [ "${@: -1}" = "-h" ] || [ "${@: -1}" = "--help" ]; then
    build_script_usage
    exit 1
  fi
  # Find <product-dir> from our arguments:
  local arg
  for arg in "$@"; do
    [[ "$arg" ==  -* ]] && continue;
    product_dir="$arg"; break
  done
  if [ -z "$product_dir" ]; then
    # Bad usage.
    ssi_die "Bad <product-dir> $product_dir\n$(build_script_usage)"
    exit 2
  elif ! { [ -d "$product_dir/.upsfiles" ] && [ -w "$product_dir" ]; }; then
    # Something wrong with <product-dir>.
    ssi_die "<product-dir> not a valid, writable unified UPS directory: " \
      "$product_dir\n$(build_script_usage)"
  fi
  # Promising...
  local abs_product_dir="$(cd "$product_dir" 2>/dev/null 2>&1 && pwd -P)"
  (( $? )) && ssi_die "unable to cd to <product-dir>: $product_dir\n" \
    "$(build_script_usage)"
  # Find our own bootstraps and yank 'em.
  local ssibuildshims_topdir="$product_dir/ssibuildshims/$ssibuildshims_version"
  [ -f "$ssibuildshims_topdir/bin/ssi_functions" ] && \
    source "$ssibuildshims_topdir/bin/ssi_functions" >/dev/null 2>&1 && \
    ensure_ups "$product_dir" || ssi_die "unable to setup UPS"
    setup ssibuildshims $ssibuildshims_version -z "$product_dir" || \
    ssi_die "unable to find ssibuildshims $ssibuildshims_version in
$product_dir and set it up with UPS."
  # Absolute is safer.
  absolute_path -- "$product_dir" product_dir
}
##################

########################################################################
# Customize-able functions.
########################################################################

##################
# Ascertain whether we built and installed correctly. Add more /
# different calls to verify_in_filesystem as appropriate. Arguments
# (e.g. -q) are passed through to verify_in_filesystem.
is_built() {
  $trace verify_in_filesystem -x "${@}" "${pkgdir}/bin/git"
}
##################

##################
# If a distribution archive is wanted and the installation is successful
# according to is_built(), then make it. Any arguments (e.g. -q) are
# passed through both to is_built() and make_basic_distribution().
#
# If make_basic_distribution() finds that the product is declared
# correctly and attempts to make the distribution, then it will cause us
# to exit with the status code of that operation. If not, it returns an
# error code.
#
# See make_basic_distribution -h for useful options to add to
# mbd_opts().
maybe_make_distribution() {
  if [ "$maketar" = "tar" ]; then
    # Customize mbd_opts.
    local mbd_opts=(-c)
    $trace is_built "$@" && \
      $trace make_basic_distribution "$@" "${mbd_opts[@]}"
  fi
}
##################

########################################################################
# Package-specific functions.
########################################################################

########################################################################
# Main.
########################################################################

####################################
# Useful variables.
trace=maybe_xtrace # Could set to xtrace instead for unconditional.
thisdir=$(cd "${BASH_SOURCE%/*}" && pwd -P)
(( ssi_trace = 1 )) # Trace certain commands by default.
####################################

####################################
# Package name and version information.
. $thisdir/version_info.sh || \
  ssi_die "unable to load version info from $thisdir/version_info"
####################################

####################################
# Handle options and arguments.

# Set up ssibuildshims and UPS / product_dir as early as possible.
ensure_ssibuildshims "$@"

##################
# Now parse options and arguments, setting basequal, extraqual, maketar,
# etc.
#
# Use verify_quals() to parse command line options, verify that the user
# has provided qualifiers supported by this build script, and set
# expected variables such as basequal, extraqual, cqual, etc.
#
# Customize the extra usage text as appropriate. Note that DEBUG_PATCHES
# and VERBOSE_PATCHES are honored by prepare_source() and
# patch_source(). DEBUG_(CONFIG|BUILD|TEST|INSTALL) should be accounted
# for (or not) by the relevant command in this script.
#
# See verify_quals -h for options and examples.
verify_quals \
  --parse --usage-file=- -- "$@" <<EOF || exit
ENVIRONMENT

  DEBUG_PATCHES
  VERBOSE_PATCHES

    Command tracing and increased verbosity for source unpacking and patching
    operations. DEBUG_PATCHES will cause exit after patches have been applied.
EOF
##################

##################
# Define globals: pkgdir, sourcedir, blddir, fullqual, etc. See
# define_basics -h for steering options.
$trace define_basics TRACK_PROGRESS NO_QUAL IN_SOURCE || exit
##################

####################################
# Declare ourselves privately, and then set ourselves up to allow setup
# of dependencies.
#
# Comment or delete if not required:
#$trace fake_declare_product "${ssi_args[@]}" -x -- || exit
####################################

####################################
# Build-only dependencies.
####################################

# Print active products.
echo
ups active
echo

# Possible short-circuit if we're already built and declared.
$trace maybe_make_distribution --silent

ssi_info "building $package for $OS-$plat-$qualdir (flavor $pkgflvr)"

# Ensure directories: pkgdir, sourcedir, blddir.
$trace init_pkg_dirs || exit

####################################
# Prepare source for building.

$trace cd "$sourcedir" \
  || ssi_die "unable to cd to source directory $sourcedir"

# Source tarballs and patches. See prepare_source -h / patch_source -h
# for options.
$trace prepare_source "${ssi_args[@]}" -P "$patchdir" \
  "$tardir/$pkgtarfile" \
  || exit

# Bail early if DEBUG_PATCHES is set.
[ -n "$DEBUG_PATCHES" ] && exit 1
####################################

####################################
# Prepare to build.
#
# Set configure / compile flags, environment variables, etc.

asciidoc="$(type -p asciidoc 2>/dev/null)"

make_cmd=(make -j$ncores NO_INSTALL_HARDLINKS=1)

##################
# Configure parallelism.
ncores=$(ncores)
##################
####################################

####################################
# Configure, build, and install.
#
# Use $blddir/$pkgtarname for in-source builds, $blddir otherwise.

$trace cd "$blddir/$pkgtarname" || \
  ssi_die "unable to cd to build directory $blddir"

##################
# Configure.
$trace autoconf ${DEBUG_CONFIG:+-v} || \
  ssi_die "unable to generate configure script"

$trace ./configure ${DEBUG_CONFIG:+-v} \
         --prefix="$pkgdir" \
         --with-libpcre \
         --with-tcltk --with-openssl --with-curl --with-expat \
  || ssi_die "configure failed"

$trace cp -v "$patchdir/config.mak" .
##################

##################
# Build.
$trace "${make_cmd[@]}" ${DEBUG_BUILD:+V=1} all || \
  ssi_die "build failed"
##################

##################
# Test.
##################

##################
# Install.
$trace "${make_cmd[@]}" ${DEBUG_INSTALL:+V=1} \
         install ${asciidoc:+install-doc} && \
  mkdir -p "$pkgtop"/etc && \
  cp contrib/completion/* "$pkgtop"/etc/ \
    || ssi_die "install failed"
##################

ssi_info "finished building $package $pkgver"

####################################
# Pre-declaration processing.
#
# Move things, install from contrib, clean build products from in-source
# builds, etc.
####################################

##################
# Declare and set up the installed package.
#
# Add -c for current declaration if required.
$trace declare_product -c -x -- || exit
##################

####################################
# Post-declaration processing.
#
# Operations requiring the *final* declaration and setup (as opposed to
# the pre-build fake declaration and setup). This should be rare.
####################################

#################
# Verify installation and make distribution archive.
#
# To customize verification, see is_built(), since it is also used by
# maybe_make_distribution(). See the documentation for those functions
# (above) for customization opportunities.

# Add a message to ssi_build_failed if appropriate.
$trace is_built && $trace ssi_build_completed && \
  ssi_info "$package is installed at $pkgdir" && \
  $trace maybe_make_distribution || \
    ssi_build_failed
########################################################################
# Nothing beyond this point.
########################################################################
